//-----------------------------------------------------------------------------------------------------------------------------
// X264Encoder / implementacia triedy
//-----------------------------------------------------------------------------------------------------------------------------

#include "x264encoder.h"
#include <cstring>
#include <iostream>
//-----------------------------------------------------------------------------------------------------------------------------

X264Encoder::X264Encoder()
{
    width = height = 0;
    fps = 25;
    fp = NULL;
    encoder = NULL;
    num_nals = 0;
    pts = 0;
}
//-----------------------------------------------------------------------------------------------------------------------------

X264Encoder::~X264Encoder()
{
    close();
}
//-----------------------------------------------------------------------------------------------------------------------------

bool X264Encoder::open(std::string filename, int width, int height, int fps)
{
    this->width = width;
    this->height = height;
    this->fps = fps;

    int r = 0;
    int nheader = 0;
    int header_size = 0;

    if (encoder)
    {
        std::cerr << "Already opened. First call close()";
        return false;
    }

    fp = fopen(filename.c_str(), "w+b");
    if (!fp)
    {
        std::cerr << "Cannot open the h264 destination file";
        goto error;
    }

    x264_picture_alloc(&pic_in, X264_CSP_YV12, width, height);
    std::cout << pic_in.img.i_csp;

    setParams();

    // create the encoder using our params
    encoder = x264_encoder_open(&params);
    if (!encoder)
    {
        std::cerr << "Cannot open the encoder";
        goto error;
    }

    // write headers
    r = x264_encoder_headers(encoder, &nals, &nheader);
    if (r < 0)
    {
        std::cerr << "x264_encoder_headers() failed";
        goto error;
    }

    header_size = nals[0].i_payload + nals[1].i_payload + nals[2].i_payload;
    if(!fwrite(nals[0].p_payload, header_size, 1, fp))
    {
        std::cerr << "Cannot write headers";
        goto error;
    }

    pts = 0;

    return true;

error:
    close();
    return false;
}
//-----------------------------------------------------------------------------------------------------------------------------

bool X264Encoder::encode(char *data)
{
    // copy the pixels into our "raw input" container.
    std::cout << pic_in.img.i_csp;  // this line fire SIGSEGV - for pic_in debuger peak "NO SUCH VALUE" ???
    pic_in.img.plane[0] = (uint8_t*)data;
    pic_in.img.plane[1] = pic_in.img.plane[0] + width * height;
    pic_in.img.plane[2] = pic_in.img.plane[1] + width * height / 4;

    // and encode and store into pic_out
    pic_in.i_pts = pts;

    int frame_size = x264_encoder_encode(encoder, &nals, &num_nals, &pic_in, &pic_out);
    if (frame_size)
    {
        if(!fwrite(nals[0].p_payload, frame_size, 1, fp))
        {
            std::cerr << "Error while trying to write nal";
            return false;
        }
    }
    ++pts;

    return true;
}
//-----------------------------------------------------------------------------------------------------------------------------

bool X264Encoder::close()
{
    if (encoder)
    {
        x264_picture_clean(&pic_in);
        memset((char*)&pic_in, 0, sizeof(pic_in));
        memset((char*)&pic_out, 0, sizeof(pic_out));

        x264_encoder_close(encoder);
        encoder = NULL;
    }
    if (fp)
    {
        fclose(fp);
        fp = NULL;
    }
    return true;
}
//-----------------------------------------------------------------------------------------------------------------------------

void X264Encoder::setParams()
{
    x264_param_default_preset(&params, "ultrafast", "zerolatency");
    params.i_threads = 1;
    params.i_width = width;
    params.i_height = height;
    params.i_fps_num = fps;
    params.i_fps_den = 1;
}
//-----------------------------------------------------------------------------------------------------------------------------
